package Emoji;

#-------------------------------------------------------------------------------
# Emoji.pm ver 0.04
# by Michiya Honda <pia@kreis-net.jp>
#-------------------------------------------------------------------------------
# $Id: Emoji.pm,v 1.1.1.1 2003/08/29 07:37:13 pia Exp $
#-------------------------------------------------------------------------------
# ver 0.04 2002/02/07
#     0.03 2001/12/22
#     0.02 2001/09/01
#     0.01 2001/08/29
#-------------------------------------------------------------------------------

use strict;
use vars qw($VERSION);
use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);

require Exporter;
@ISA			= qw(Exporter);
@EXPORT			= qw(imode_encode imode_decode jsky_encode jsky_decode);
@EXPORT_OK		= qw();
%EXPORT_TAGS	= ( all => [ @EXPORT, @EXPORT_OK ] );

$VERSION		= '0.04';

#-------------------------------------------------------------------------------

sub new {
	my $class = shift;
	my $original = shift;;
	
	return bless {
		original	=> $original,
		modified	=> '',
	}, $class;
}

#-------------------------------------------------------------------------------

sub _accessor {
	my($self, $name, $data) = @_;
	$self->{$name} = $data if (@_ >= 3);
	$self->{$name};
}

#-------------------------------------------------------------------------------

sub original	{ my $self = shift; return $self->_accessor('original',	@_); }
sub modified	{ my $self = shift; return $self->_accessor('modified',	@_); }

#-------------------------------------------------------------------------------

sub hexdec { hex($_[0]); }
sub dechex { sprintf("%lx",$_[0]); }

#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
# $obj->imode_encode($opt,$replace);
#   $opt     : true      : detect i-mode emoji and convert HTML entitiy
#              false     : detect i-mode emoji and remove it
#   $replace : with $opt : detect i-mode emoji and replace it with $replace
#-------------------------------------------------------------------------------

sub imode_encode {
	my $self = shift;
	my $opt = shift;
	my $replace = shift;
	
	my $str = $self->original;
	my $converted = '';
	my @str_array = unpack("C*",$str);
	
	for (my $i = 0;$i < @str_array;$i++){
		my $pstr = pack("C",$str_array[$i]);
		if ($pstr =~ /^[\x00-\x7F\xA1-\xDF]$/) {
			$converted .= $pstr;
		}
		elsif ($str_array[$i] == 0xF8) {
			my $pstr1 = pack("C",$str_array[$i+1]);
			if ($pstr1 =~ /^[\x9F-\xFC]$/) {
				if ($opt) {
					if (defined $replace) {
						$converted .= $replace;
					}
					else {
						my $hex = dechex($str_array[$i]) . dechex($str_array[$i+1]);
						$converted .= '&#' . hexdec($hex) . ';';
					}
				}
			}
			else {
				$converted .= pack("C2",$str_array[$i],$str_array[$i+1]);
			}
			$i++;
		}
		elsif ($str_array[$i] == 0xF9) {
			my $pstr1 = pack("C",$str_array[$i+1]);
			if ($pstr1 =~ /^[\x40-\x49\x50-\x53\x55-\x57\x5B-\x7E\x80-\xB0]$/) {
				if ($opt) {
					if (defined $replace) {
						$converted .= $replace;
					}
					else {
						my $hex = dechex($str_array[$i]) . dechex($str_array[$i+1]);
						$converted .= '&#' . hexdec($hex) . ';';
					}
				}
			}
			else {
				$converted .= pack("C2",$str_array[$i],$str_array[$i+1]);
			}
			$i++;
		}
		else {
			$converted .= pack("C2",$str_array[$i],$str_array[$i+1]);
			$i++;
		}
	}
	
	return $self->modified($converted);
}

#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
# $obj->imode_decode($opt,$replace);
#   $opt     : true      : detect i-mode emoji HTML entitiy and convert raw text
#              false     : detect i-mode emoji HTML entitiy and remove it
#   $replace : with $opt : detect i-mode emoji HTML entitiy and replace it with $replace
#-------------------------------------------------------------------------------

sub imode_decode {
	my $self = shift;
	my $opt = shift;
	my $replace = shift;
	
	my $str = $self->original;
	my $converted = '';
	my @str_array = unpack("C*",$str);
	
	for (my $i = 0;$i < @str_array;$i++){
		if ($str_array[$i] == 0x26 && $str_array[$i+1] == 0x23) {
			my $tmp = '';
			$i += 2;
			while ($str_array[$i] != 0x3B) {
				$tmp .= pack("C",$str_array[$i]);
				$i++;
			}
			if (
				($tmp >= 0xF89F && $tmp <= 0xF8FC) ||
				($tmp >= 0xF940 && $tmp <= 0xF949) ||
				($tmp >= 0xF950 && $tmp <= 0xF953) ||
				($tmp >= 0xF955 && $tmp <= 0xF957) ||
				($tmp >= 0xF95B && $tmp <= 0xF97E) ||
				($tmp >= 0xF980 && $tmp <= 0xF9B0)
			) {
				if ($opt) {
					my $dec1 = int($tmp / 256);
					my $dec2 = $tmp % 256;
					$converted .= pack("C2",$dec1,$dec2);
				}
			}
			else {
				$converted .= '&#' . $tmp . ';';
			}
		}
		else{
			$converted .= pack("C",$str_array[$i]);
		}
	}
	
	return $self->modified($converted);
}

#-------------------------------------------------------------------------------
# $obj->jsky_encode($opt,$replace);
#   $opt     : true      : DO NOTHING
#              false     : detect J-sky emoji and remove it
#   $replace : with $opt : detect J-sky emoji and replace it with $replace
#-------------------------------------------------------------------------------

sub jsky_encode {
	my $self = shift;
	my $opt = shift;
	my $replace = shift;
	
	my $str = $self->original;
	
	if ($opt) {
		if (defined $replace) {
			$str =~ s/\x1B\x24[E-G].*?\x0F/$replace/eg;
		}
		else {
			$str =~ s/\x1B\x24([E-G].*?)\x0F/\&\#27\;\&\#36\;$1\&\#15\;/g;
		}
	}
	else {
		$str =~ s/\x1B\x24[E-G].*?\x0F//g;
	}
	
	return $self->modified($str);
}

#-------------------------------------------------------------------------------
# $obj->jsky_decode($opt,$replace);
#   $opt     : true      : DO NOTHING
#              false     : detect J-sky emoji HTML entitiy and remove it
#   $replace : with $opt : detect J-sky emoji HTML entitiy and replace it with $replace
#-------------------------------------------------------------------------------

sub jsky_decode {
	my $self = shift;
	my $opt = shift;
	my $replace = shift;
	
	my $str = $self->original;
	my $converted = '';
	my @str_array = unpack("C*",$str);
	
	for (my $i = 0;$i < @str_array;$i++){
		if ($str_array[$i] == 0x26 && $str_array[$i+1] == 0x23) {
			my $tmp = '';
			$i += 2;
			while ($str_array[$i] != 0x3B) {
				$tmp .= pack("C",$str_array[$i]);
				$i++;
			}
			if ( ($tmp == 27) || ($tmp == 36) || ($tmp == 15) ) {	# '&' or '#' or ';'
				if ($opt) {
					$converted .= pack("C",$tmp);
				}
			}
			else {
				$converted .= '&#' . $tmp . ';';
			}
		}
		else{
			$converted .= pack("C",$str_array[$i]);
		}
	}
	
	return $self->modified($converted);
}

#-------------------------------------------------------------------------------
# $obj->imode_safe;
#-------------------------------------------------------------------------------

sub imode_safe {
	my $self = shift;
	my $opt = shift;
	my $replace = shift;
	
	$self->original($self->jsky_encode($opt,$replace));
	
	return $self->modified;
}

#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
# $obj->jsky_safe;
#-------------------------------------------------------------------------------

sub jsky_safe {
	my $self = shift;
	my $opt = shift;
	my $replace = shift;
	
	$self->original($self->imode_encode($opt,$replace));
	
	return $self->modified;
}

#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
# $obj->ezweb_safe;
#-------------------------------------------------------------------------------

sub ezweb_safe {
	my $self = shift;
	my $opt = shift;
	my $replace = shift;
	
	$self->original($self->jsky_encode($opt,$replace));
	$self->original($self->imode_encode($opt,$replace));
	
	return $self->modified;
}

#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
# $obj->all_safe;
#-------------------------------------------------------------------------------

sub all_safe {
	my $self = shift;
	
	$self->original($self->jsky_encode());
	$self->original($self->imode_encode());
	
	return $self->modified;
}

#-------------------------------------------------------------------------------

1;
__END__

=head1 NAME

Emoji - Perl extension for the 'emoji' of Japanese cellular phone

=head1 SYNOPSIS

  use Emoji;
  
  # Shift_JIS Characters for Japanese 'i-mode' cellular phone
  # "\xF9\x91" is a heart mark
  my $imode_str = "\xF9\x91";
  
  # encoding. :-)
  my $imode_encoded = Emoji->new($imode_str)->imode_encode(1);
  
  # decoding. :-)
  my $imode_decoded = Emoji->new($imode_encoded)->imode_decode(1);
  
  # Shift_JIS Characters for Japanese 'J-sky' cellular phone
  # "\x1B\x24GB\x0F" is a heart mark
  my $jsky_str = "\x1B\x24GB\x0F";
  
  # encoding. :-)
  my $jsky_encoded = Emoji->new($jsky_str)->jsky_encode(1);
  
  # decoding. :-)
  my $jsky_decoded = Emoji->new($jsky_encoded)->jsky_decode(1);
  
  # Shift_JIS Characters for Japanese 'ezWeb' cellular phone
  # This is dangerous chars what causes error ...
  my $ezweb_str = "\xF9\x91\x1B\x24GB\x0F";
  
  # replacing safely. :-)
  my $ezweb_safestr = Emoji->new($ezweb_str)->ezweb_safe(1,'[XXX]');
  
  print 'i-mode heart mark : [', $imode_encoded, ']', "\n";
  print 'i-mode heart mark : [', $imode_decoded, ']', "\n";
  print "\n";
  print 'j-sky heart mark : [', $jsky_encoded, ']', "\n";
  print 'j-sky heart mark : [', $jsky_decoded, ']', "\n";
  print "\n";
  print 'ezweb dangerous chars : [', $ezweb_safestr, ']', "\n";

=head1 DESCRIPTION

This module handles safely the special characters for cellular phone - i-mode, J-sky, ezweb.
See http://www.nttdocomo.co.jp/i/tag/emoji/index.html for details about i-mode emoji.

=head1 FUNCTIONS

For using all functions in this module, string parameters must be encoded in Shift_JIS.
More details about Shift_JIS, see L<Jcode>.

=over 4

=item imode_encode

  $encoded = $obj->imode_encode($opt,$replace);

Detects i-mode emoji characters, converts HTML entities, and remove it if you need.

=item imode_decode

  $decoded = $obj->imode_decode($opt,$replace);

Detects i-mode emoji HTML entities, converts raw text, and remove it if you need.

=item jsky_encode

  $encoded = $obj->jsky_encode($opt,$replace);

Detects J-sky emoji characters, converts HTML entities, and remove it if you need.

=item jsky_decode

  $decoded = $obj->jsky_decode($opt,$replace);

Detects J-sky emoji characters, converts HTML entities, and remove it if you need.

=item imode_safe

  $safe = $obj->imode_safe($opt,$replace);

Remains to only i-mode characters, and converts/remove other emoji characters.

=item jsky_safe

  $safe = $obj->jsky_safe($opt,$replace);

Remains to only J-sky characters, and converts/remove other emoji characters.

=item ezweb_safe

  $safe = $obj->ezweb_safe($opt,$replace);

Remains to only ezweb characters, and converts/remove other emoji characters.

=item all_safe

  $safe = $obj->all_safe;

Converts/remove all emoji characters.

=back

=head1 CAVEAT

This module is still beta (less than 1.00) version.
Therefore it is welcome to send bug reports to me.
Please use this at your own risk.

=head1 AUTHOR INFO

Copyright 2001 Michiya Honda <pia@kreis-net.jp>, All rights reserved.

=head1 CREDITS

Thanks very much to:

=over 4

=item Tatsuhiko Miyagawa <miyagawa@bulknews.net>

=item Kenichi Kimura <lovex@bd.mbn.or.jp>

for hints, suggestions and bug fixes.

=back

=head1 SEE ALSO

L<HTML::Entities>, L<HTML::Entities::ImodePictogram>, http://www.nttdocomo.co.jp/i/tag/emoji/index.html

=cut
1;
__END__
